import java.io.*;

class Nodo {
    private Point point;
    private Nodo link;

    public Nodo() {
        this.point = new Point();
        this.link = null;
    }

    public Nodo(Point point, Nodo link) {
        this.point = point;
        this.link = link;
    }

    public Nodo (Nodo n) {
        this.point = new Point(n.getPoint());
        this.link = n.getLink();
    }

    public void setLink(Nodo link) {
        this.link = link;
    }

    public void setPoint(Point point) {
        this.point = new Point(point);
    }

    public Nodo getLink() {
        return link;
    }

    public Point getPoint() {
        Point p = new Point(point);
        return p;
    }
}

class PolylineException extends Exception {    
}




public class Polyline {
    private int elementi;
    private Nodo head;
    
    public Polyline () {
        elementi = 0;
        head = null;
    }
    
    public Polyline (Point[] nodi) {
        this.elementi = 0;
        head = null;
        
        for (int n=0; n<nodi.length; n++) {
            try {
                addPoint(nodi[n],n);
            }
            catch (PolylineException exception) {
                // to do
            }
        }
    }
    
    private Nodo getHead() {
        return head;
    }
    
    public int getNodi() {
        return elementi;
    }
    
    public void addPoint(Point p, int i) throws PolylineException {
        Nodo nodo, nuovo = new Nodo(p,null);
        
        if ((i<0) || (i>elementi)) throw new PolylineException();
        
        if (i==0) {
            nuovo.setLink(head);
            head = nuovo;
            elementi++;
            return;
        }
        nodo = head;
        for (int n=1; n<i; n++) {
            nodo = nodo.getLink();
        }
        nuovo.setLink(nodo.getLink());
        nodo.setLink(nuovo);
        elementi++;
        return;
    }
    
    public void delPoint(int i) throws PolylineException {
        Nodo precedente,nodo;
        
        if ((i<=0) || (i>elementi)) throw new PolylineException();
        
        if (i==1) {
            head = head.getLink();
            elementi--;
            return;
        }
        precedente = head;
        nodo = head.getLink();
        for (int n=1; n<i; n++) {
            precedente = nodo;
            nodo = nodo.getLink();
        }
        precedente.setLink(nodo.getLink());
        elementi--;
        return;
    }
    
    public double computeLen() throws PolylineException {
        double len = 0.0;
        Nodo precedente,nodo;
        
        if (elementi==0) throw new PolylineException();
        if (elementi==1) return 0.0;
        
        precedente = head;
        nodo = head.getLink();
        while (nodo.getLink() != null) {
            len += precedente.getPoint().distance(nodo.getPoint());
            precedente = nodo;
            nodo = nodo.getLink();
        }
        len += precedente.getPoint().distance(nodo.getPoint());
        return len;
    }
    
    public Point[] closureRectangle() throws PolylineException {
        Nodo nodo;
        Point[] vertex = new Point[2];
        float Xmin,Xmax,Ymin,Ymax;
        
        if (elementi==0) throw new PolylineException();
        
        vertex[0] = new Point();
        vertex[1] = new Point();

        Xmin = head.getPoint().getX();
        Xmax = head.getPoint().getX();
        Ymin = head.getPoint().getY();
        Ymax = head.getPoint().getY();
        nodo = head.getLink();
        while (nodo.getLink() != null) {
            if (nodo.getPoint().getX() < Xmin)
                Xmin = nodo.getPoint().getX();
            if (nodo.getPoint().getX() > Xmax)
                Xmax = nodo.getPoint().getX();
            if (nodo.getPoint().getY() < Ymin)
                Ymin = nodo.getPoint().getY();
            if (nodo.getPoint().getY() > Ymax)
                Ymax = nodo.getPoint().getY();
            nodo = nodo.getLink();
        }
        vertex[0].setX(Xmin);
        vertex[0].setY(Ymin);
        vertex[1].setX(Xmax);
        vertex[1].setY(Ymax);
               
        return vertex;
    } 
    
    public void chainPolyline(Polyline pl) {
        Nodo nodo;
        
        if (elementi==0) {
            head = pl.getHead();
            elementi += pl.getNodi();
            return;
        }
        nodo = head;
        while (nodo.getLink() != null)
            nodo = nodo.getLink();
        nodo.setLink(pl.getHead());
        elementi += pl.getNodi();
        return;
    }

    public void saveToFile() throws IOException {
        TextFile out = new TextFile("polyline.txt",'W');
        Nodo nodo = head;

        while (nodo.getLink() != null) {
            try {
                out.toFile(nodo.getPoint().toString());
            }
            catch (FileException e) {
                // code never trap here!
            }
            nodo = nodo.getLink();
        }
        try {
                if (nodo != null)
                    out.toFile(nodo.getPoint().toString());
            }
            catch (FileException e) {
                // code never trap here!
            }
        out.closeFile();
    }

    public void loadFromFile() throws IOException {
        TextFile in = new TextFile("polyline.txt",'R');
        String line, substring[];
        this.elementi = 0;
        head = null;
        Point p;
        int pos = 0;
        float x,y;

        try {
            while (true) {
                line = in.fromFile();
                substring = line.split(";");
                x = Float.parseFloat(substring[0]);
                y = Float.parseFloat(substring[1]);
                p = new Point(x, y);
                addPoint(p,pos);
                pos++;
            }
        }
        catch (FileException e) {
            in.closeFile();
        }
        catch (PolylineException exception) {
                // to do
            }
    }

    public static void main(String args[]) throws PolylineException, IOException {
        Point A = new Point(0,0);
        Point B = new Point(1,0);
        Point C = new Point(1,1);
        Point D = new Point(0,1);
              
        Point[] points = new Point[2];
        points[0] = new Point(-1,1);
        points[1] = new Point(-1,0);
        
        Polyline pl1 = new Polyline();
        pl1.addPoint(A, 0);
        pl1.addPoint(B, 1);
        pl1.addPoint(C, 2);
        pl1.addPoint(D, 3);
        pl1.saveToFile();
        Polyline pl2 = new Polyline(points);
                
        System.out.println(pl1.computeLen());
        points = pl1.closureRectangle();
        System.out.println(points[0]);
        System.out.println(points[1]);
        pl1.delPoint(2);
        System.out.println(pl1.computeLen());
        pl1.delPoint(2);
        pl1.loadFromFile();
        System.out.println(pl1.computeLen());
        pl1.chainPolyline(pl2);
        System.out.println(pl1.computeLen());
        points = pl1.closureRectangle();
        System.out.println(points[0]);
        System.out.println(points[1]);
    }
}
