public class TabellaAcquisti {
 private Prodotto acquisti[]; // tabella
 private int numero_prodotti;
 
 public TabellaAcquisti(int dimensione) {
   acquisti = new Prodotto[dimensione];
   numero_prodotti=0;
 }
// funzione hash
 private int hash (String chiave) {
   int p = 0;
   for (int i=0; i<chiave.length(); i++) {
     p += chiave.charAt(i);
   }
   return (p % acquisti.length);
 }
 
// operazione R = ricerca per informazioni
// operazione I = ricerca per inserimento
 private int cercaProdotto(String chiave, char operazione) throws TabellaException {
     int p = hash(chiave);
     if (operazione == 'R') {
     // ricerca per informazioni
      if (acquisti[p] == null)
       throw new TabellaException( "Elemento "+chiave+ " non esistente");
      if (acquisti[p].getCodice_barre().equals(chiave))
       return p;
     }
     else {
     // ricerca per inserimento
      if (acquisti[p] == null)
       return p;
      if (acquisti[p].getCodice_barre().equals(chiave))
        throw new TabellaException( "Elemento "+acquisti[p].getCodice_barre()+ " preesistente");
     }
     // collisione -> scansione lineare
     p = scansioneLineare(p, chiave, operazione);
     return p;
 }
 public Prodotto cerca(String codice_barre) throws TabellaException {
   int p = cercaProdotto(codice_barre,'R');
   return new Prodotto(acquisti[p]);
 }
 public int getNumeroProdotti() {
  return numero_prodotti;
 }
 public int inserisciProdotto(Prodotto prodotto, int quantita) throws TabellaException {
  String chiave = prodotto.getCodice_barre();
  int p = cercaProdotto(chiave, 'I');
  acquisti[p] = new Prodotto(prodotto);
  acquisti[p].setQuantita(quantita);
  numero_prodotti++;
  return p;
}
public int eliminaProdotto(String codice_barre) throws TabellaException {
  int p = cercaProdotto(codice_barre, 'R');
  acquisti[p] = null;
  numero_prodotti--;
  return p;
}
// funzione di visualizzazione dello stato corrente
// utilizzata a scopo di test
public void visualizzaStatoTabella() {
  for (int i=0; i<acquisti.length; i++)
  if (acquisti[i] != null)
  System.out.println("Posizione "+i+"->"+acquisti[i].toString()+"(hash: "+hash(acquisti[i].getCodice_barre())+")");
}

public double importoTotale(){
  double imp_tot=0.0;
  for (int i=0; i<acquisti.length; i++)
  if (acquisti[i] != null)
    imp_tot+=acquisti[i].getPrezzo()*acquisti[i].getQuantita();
  return imp_tot;
}
private int scansioneLineare ( int posizioneIniziale, String chiave, char operazione) throws TabellaException {
  int p = posizioneIniziale;
  while (true) {
   p++;
   p = p % acquisti.length;
   if (operazione == 'R') {
   // ricerca per informazione
    if (acquisti[p] == null)
     throw new TabellaException( "Elemento "+chiave+ " non esistente");
    if (acquisti[p].getCodice_barre().equals(chiave))
    return p;
   }
   else {
   // ricerca per inserimento
    if (acquisti[p] == null)
    return p;
    if (acquisti[p].getCodice_barre().equals(chiave))
      throw new TabellaException( "Elemento "+acquisti[p].getCodice_barre()+ " preesistente");
  }
  if (p == posizioneIniziale)
   if (operazione == 'R')
    throw new TabellaException( "Elemento "+chiave+ " non esistente");
   else
   throw new TabellaException("Tabella piena");
  }
}
public static void main (String[] args) {
    //String codice_barre, String descrizione, double prezzo, int quantita
 Prodotto p1 = new Prodotto("ABC-efg-1234","Mele",2,0);
 Prodotto p2 = new Prodotto("AAA-abc-2234","Pasta",3,0);
 Prodotto p3 = new Prodotto("CAX-cba-5681","Cioccolata",4,0);
 Prodotto p4 = new Prodotto("ABB-xyz-2367","Pane",1,0);
 Prodotto p5 = new Prodotto("AXY-jbe-3334","Prosciutto",5,0);
 TabellaAcquisti t = new TabellaAcquisti(100);
 try {
   System.out.println("Inserimento prodotto " +p1.getDescrizione()+" posizione: "+t.inserisciProdotto(p1,5));
   System.out.println("Inserimento prodotto " +p2.getDescrizione()+" posizione: "+t.inserisciProdotto(p2,2));
   System.out.println("Inserimento prodotto " +p3.getDescrizione()+" posizione: "+t.inserisciProdotto(p3,4));
   System.out.println("Inserimento prodotto " +p4.getDescrizione()+" posizione: "+t.inserisciProdotto(p4,1));
   System.out.println("Inserimento prodotto " +p5.getDescrizione()+" posizione: "+t.inserisciProdotto(p5,2));
 }
 catch (TabellaException e) {
  System.out.println(e.getError());
 }
 System.out.println("Stato tabella:");
 t.visualizzaStatoTabella();
 System.out.println("Importo totale spesa: " + t.importoTotale()+"euro\n");
 try {
   System.out.println("Ricerca AAA-abc-2234: "+ t.cerca("AAA-abc-2234"));
   System.out.println("Ricerca AAA-abd-2234: "+ t.cerca("AAA-abd-2234"));
 }
 catch (TabellaException e) {
   System.out.println(e.getError());
 }
 try {
   System.out.println( "Eliminazione prodotto ABB-xyz-2367 in posizione: "+t.eliminaProdotto("ABB-xyz-2367"));
   System.out.println( "Eliminazione prodotto CCC-cba-5681 in posizione: "+t.eliminaProdotto("CCC-cba-5681"));
 }
 catch (TabellaException e) {
   System.out.println(e.getError());
 }
 System.out.println("Stato tabella:");
 t.visualizzaStatoTabella();
 System.out.println("Importo totale spesa: " + t.importoTotale()+"euro\n");
 }

}